/*
The Mecanum-wheel AI robot "DanVI,"
Developed by: Metaplace Co., Ltd.(T.033-252-4787)
BuiltIn(ATMEGA32U4-MU) 
PCB_ver 2.0
SMC(6850)MOTER
*/

// 디지털핀 번호 정의
// LED가 연결된 핀 번호는 디지털 핀 13번으로 설정합니다.
#define led_pin 13
// Buzzer(부저)가 연결된 핀번호는 디지털핀 6번으로 설정합니다. 
#define buzzer_pin 6                  

// 아날로그핀 번호 정의
// 푸쉬버튼이 연결된 핀 번호는 아날로그 핀 A0번으로 설정
#define button_pin A0

// 포토 저항 연결된 핀 번호는 아날로그 핀 A3번으로 설정
#define photoR_pin A3

// 버튼 상태를 저장할 변수 선언
// button_state :: 푸쉬스위치의 현재 상태를 저장하는 변수로, 0은 꺼짐(OFF), 1은 켜짐(ON)을 의미한다.
int button_state = 0;

// setup() 함수는 부팅 시에만 한 번 실행되는 함수입니다.
void setup()
{
  // 시리얼 통신 초기화: 컴퓨터와의 데이터 통신 속도를 9600bps(bits per second)로 설정합니다.
  // 이 설정은 Serial Monitor에서 데이터를 송수신할 때 사용됩니다.
  Serial.begin(9600);

  // 디지털 핀 모드 설정: led_pin(13번 핀)과 buzzer_pin(6번 핀)을 출력 모드(OUTPUT)로 설정합니다.
  // 이 핀들은 LED와 부저(Buzzer)를 제어하는 데 사용됩니다.
  pinMode(led_pin, OUTPUT);
  pinMode(buzzer_pin, OUTPUT);
}
// setup() 함수 종료

// loop() 함수는 메인 프로그램으로, Arduino가 켜져 있는 동안 계속 반복 실행됩니다.
void loop()
{
  // 아날로그 입력값 읽기: 포토 저항 확인하기 위해 photoR_pin(A3)에서 값을 읽어옵니다.
  // 아날로그 입력값은 0에서 1023 사이의 값을 가질 수 있으며, 
  // 포토 저항 주변이 밝으면 512이상, 어두우면 512이하의 값이 생성됩니다.

  Serial.println(analogRead(photoR_pin));

  // 포토 저항 주변이 밝으면 512이상, 어두우면 512이하
  if (analogRead(photoR_pin) < 400)
  {
    // LED ON: led_pin(13번 핀)에 HIGH 신호를 보내 LED를 켭니다.
    digitalWrite(led_pin, HIGH);
    // 시리얼 모니터에 "LED ON" 메시지를 출력합니다.
    Serial.println("LED ON");

    // BUZZER ON: buzzer_pin(6번 핀)에 HIGH 신호를 보내 부저(Buzzer)를 켭니다.
    digitalWrite(buzzer_pin, HIGH);
    // 시리얼 모니터에 "BUZZER ON" 메시지를 출력합니다.
    Serial.println("BUZZER ON");

  }
  // 포토 저항 주변이 밝으면 512이상, 어두우면 512이하
  else if (analogRead(photoR_pin) > 400)
  {
    // LED OFF: led_pin(13번 핀)에 LOW 신호를 보내 LED를 끕니다.
    digitalWrite(led_pin, LOW);
    // 시리얼 모니터에 "LED OFF" 메시지를 출력합니다.
    Serial.println("LED OFF");
    
    // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
    digitalWrite(buzzer_pin, LOW);
    // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
    Serial.println("BUZZER OFF");
  }   
}
// loop() 함수 종료

