/*
The Mecanum-wheel AI robot "DanVI,"
Developed by: Metaplace Co., Ltd.(T.033-252-4787)
BuiltIn(ATMEGA32U4-MU) 
PCB_ver 2.0
SMC(6850)MOTER
*/

// 모터 설정 파일을 포함해요
#include "motor_setting.h" 

// 디지털핀 번호 정의
// LED가 연결된 핀 번호는 디지털 핀 13번으로 설정합니다.
#define led_pin 13
// Buzzer(부저)가 연결된 핀번호는 디지털핀 6번으로 설정합니다. 
#define buzzer_pin 6                  

// 아날로그핀 번호 정의
// 푸쉬버튼이 연결된 핀 번호는 아날로그 핀 A0번으로 설정
#define button_pin A0

// 버튼 상태를 저장할 변수 선언
// button_state :: 푸쉬스위치의 현재 상태를 저장하는 변수로, 0은 꺼짐(OFF), 1은 켜짐(ON)을 의미한다.
int button_state = 0;

// 부팅 시에만 실행되는 setup() 함수
void setup()
{
  // 시리얼 통신 초기화: 컴퓨터와의 데이터 통신 속도를 9600bps(bits per second)로 설정
  Serial.begin(9600);

  // 디지털 핀 초기화: LED와 BUZZER를 제어할 핀들을 출력 모드로 설정
  pinMode(led_pin, OUTPUT);           // led_pin을 출력 모드로 설정
  pinMode(buzzer_pin, OUTPUT);        // buzzer_pin을 출력 모드로 설정

  // motor_setting.h 파일에서 모터 핀을 설정하는 함수를 호출하여 모터 핀을 초기화
  motor_output();
}
// setup() 함수 종료

// loop() 함수는 메인 프로그램으로, Arduino가 켜져 있는 동안 계속 반복 실행됩니다.
void loop() {
  // 푸쉬 버튼이 눌렸는지 확인 (아날로그 값이 512 이상이면 눌린 것으로 간주)
  if (analogRead(button_pin) > 512) {
    // 노이즈 방지와 안정된 버튼 입력을 위해 500ms 동안 대기
    delay(500);

    if (button_state == 0) {  // 버튼 상태가 0(눌리지 않은 상태)일 때
      // 왼쪽으로 가는 함수 호출 (모터를 왼쪽으로 회전시킴)
      move_Leftward();
      // 모터를 3초 동안 회전시킨 후 정지
      delay(3000);
      // 모터를 정지시키는 함수 호출
      Stop_Moter();
    }
  }
}
