/*
The Mecanum-wheel AI robot "DanVI,"
Developed by: Metaplace Co., Ltd.(T.033-252-4787)
BuiltIn(ATMEGA32U4-MU) 
PCB_ver 2.0
SMC(6850)MOTER
*/

// 모터 설정 파일을 포함해요
#include "motor_pwm_setting.h"

// 디지털핀 번호 정의
// LED가 연결된 핀 번호는 디지털 핀 13번으로 설정합니다.
#define led_pin 13
// Buzzer(부저)가 연결된 핀번호는 디지털핀 6번으로 설정합니다. 
#define buzzer_pin 6                  

// 아날로그핀 번호 정의
// 푸쉬버튼이 연결된 핀 번호는 아날로그 핀 A0번으로 설정
#define button_pin A0            :: 푸쉬스위치 입력

// 버튼 상태를 저장할 변수 선언
// button_state :: 푸쉬스위치의 현재 상태를 저장하는 변수로, 0은 꺼짐(OFF), 1은 켜짐(ON)을 의미한다.
int button_state = 0;

/*
  SMC(6850) 바퀴 완조립시 rpm(목측)
  PWM(255) = 80rpm
  PWM(224) = 70rpm
  PWM(192) = 60rpm
  PWM(160) = 50rpm
*/
//위의 PWM 속도를 자유롭게 조정하여 입력해 보세요. 
int PWM = 224;

// 블루투스 받는 문자열
String bluetooth_string = "";

//-----
// 부팅시 만 실행
void setup()
{
  // 시리얼 통신 초기화: 컴퓨터와의 데이터 통신 속도를 9600bps(bits per second)로 설정
  Serial.begin(9600);

  // 디지털 핀 초기화: LED와 BUZZER를 제어할 핀들을 출력 모드로 설정
  pinMode(led_pin, OUTPUT);           // led_pin을 출력 모드로 설정
  pinMode(buzzer_pin, OUTPUT);        // buzzer_pin을 출력 모드로 설정

  // motor_setting.h 파일에서 모터 핀을 설정하는 함수를 호출하여 모터 핀을 초기화
  motor_output();

  //블루투스시리얼[하드웨어] 초기화(9600)bps(bits per second)
  Serial1.begin(9600);  
  //LED 켜기 
  digitalWrite(led_pin, HIGH); 
}
// setup() 함수 종료

// loop() 함수는 메인 프로그램으로, Arduino가 켜져 있는 동안 계속 반복 실행됩니다.
void loop() {
 // 블루투스시리얼[하드웨어] 명령 수신
  if (Serial1.available())
  { 
    // Serial1[하드웨어] 값이 있으면
    bluetooth_string = Serial1.read();    // 블루투스에서 전달된 내용 저장
    Serial.println(bluetooth_string);     //블루투스측 내용을 시리얼모니터에 출력
  }  

  // Serial.println("bluetooth_string : " + bluetooth_string);       //시리얼모니터에 bluetooth_string 값 출력
  if (bluetooth_string != "")         //bluetooth_string 값이 있다면
  { 
    // 실행(1~9)(전화버튼 기준):: ASCII

    // 1 = "49"
    if (bluetooth_string == "49")
    {
      Serial.println("왼쪽 라운딩 전진");
      move_LeftTurnRoundRearAxis(PWM);

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      
    }

    // 2 = "50"
    else if (bluetooth_string == "50")
    {
      Serial.println("전진");
      move_Forward(PWM);

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      
    }

    // 3 = "51"
    else if (bluetooth_string == "51")
    {
      Serial.println("오른쪽 라운딩 전진");
      move_RightTurnRoundRearAxis(PWM);

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      
    }

    // 4 = "52"
    else if (bluetooth_string == "52")
    {
      Serial.println("좌측이동");
      move_Leftward(PWM);

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      
    }

    // 5 = "53"
    else if (bluetooth_string == "53")
    {
      Serial.println("중지");
      Stop_Moter();

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      
    }
    
    // 6 = "54"
    else if (bluetooth_string == "54")
    {
      Serial.println("우측이동");
      move_Rightward(PWM);    

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      
    }

    // 7 = "55"
    else if (bluetooth_string == "55")
    {
      Serial.println("왼쪽 라운딩 후진");
      move_LeftTurnRoundFrontAxis(PWM);

      // BUZZER ON: buzzer_pin(6번 핀)에 HIGH 신호를 보내 부저(Buzzer)를 켭니다.
      digitalWrite(buzzer_pin, HIGH);
      // 시리얼 모니터에 "BUZZER ON" 메시지를 출력합니다.
      Serial.println("BUZZER ON");    
    }

    // 8 = "56"
    else if (bluetooth_string == "56")
    {
      Serial.println("후진");
      move_Backward(PWM);

      // BUZZER ON: buzzer_pin(6번 핀)에 HIGH 신호를 보내 부저(Buzzer)를 켭니다.
      digitalWrite(buzzer_pin, HIGH);
      // 시리얼 모니터에 "BUZZER ON" 메시지를 출력합니다.
      Serial.println("BUZZER ON");      
    }

    // 9 = "57"
    else if (bluetooth_string == "57")
    {
      Serial.println("오른쪽 라운딩 후진");
      move_RightTurnRoundFrondAxis(PWM);

      // BUZZER ON: buzzer_pin(6번 핀)에 HIGH 신호를 보내 부저(Buzzer)를 켭니다.
      digitalWrite(buzzer_pin, HIGH);
      // 시리얼 모니터에 "BUZZER ON" 메시지를 출력합니다.
      Serial.println("BUZZER ON");          
    }

  // bluetooth_string초기화
  bluetooth_string = "";
  }

  // 시리얼통신 명령 수신
  if (Serial.available()) {
    Serial1.write(Serial.read());  // 시리얼모니터 내용을 블루투스측에 출력
  }    
}