/*
The Mecanum-wheel AI robot "DanVI,"
Developed by: Metaplace Co., Ltd.(T.033-252-4787)
BuiltIn(ATMEGA32U4-MU) 
PCB_ver 2.0
SMC(6850)MOTER
*/

// 모터 설정 파일을 포함해요
#include "motor_pwm_setting.h"

// 디지털핀 번호 정의
// LED가 연결된 핀 번호는 디지털 핀 13번으로 설정합니다.
#define led_pin 13

// 아날로그핀 번호 정의
// 푸쉬버튼이 연결된 핀 번호는 아날로그 핀 A0번으로 설정
#define button_pin A0          

// 블루투스 받는 문자열
String bluetooth_string = "";
// 버튼 상태를 저장할 변수 선언
// button_state :: 푸쉬스위치의 현재 상태를 저장하는 변수로, 0은 꺼짐(OFF), 1은 켜짐(ON)을 의미한다.
int button_state = 0;

// A1번 아날로그핀(line_sensor_Left_pin) 
#define line_sensor_L_pin A1          
// A2번 아날로그핀(line_sensor_Right_pin) 
#define line_sensor_R_pin A2      

// 라인센서 변수선언
int line_sensor_L, line_sensor_R;

/*
  SMC(6850) 바퀴 완조립시 rpm(목측)
  PWM(255) = 80rpm
  PWM(224) = 70rpm
  PWM(192) = 60rpm
  PWM(160) = 50rpm
*/
//위의 PWM 속도를 자유롭게 조정하여 입력해 보세요. 
// Ver 2.0(PWM 244 권장) / Ver 3.0(PWM 192 권장)
int PWM = 192;
// 라인센서 임계값
int line_sensor_limit_L = 60;
int line_sensor_limit_R = 60;


//-----
// 부팅시 만 실행
void setup()
{
  // 시리얼 통신 초기화: 컴퓨터와의 데이터 통신 속도를 9600bps(bits per second)로 설정
  Serial.begin(9600);
  //블루투스시리얼[하드웨어] 초기화(9600)bps(bits per second)
  Serial1.begin(9600);

  // 디지털 핀 초기화: LED와 BUZZER를 제어할 핀들을 출력 모드로 설정
  pinMode(led_pin, OUTPUT);           // led_pin을 출력 모드로 설정
  // motor_setting.h 파일에서 모터 핀을 설정하는 함수를 호출하여 모터 핀을 초기화
  motor_output();
  //LED 켜기 
  digitalWrite(led_pin, HIGH);  
}
// 부팅시 만 실행


//-----
// 메인(반복루프)
void loop()
{
  // Line_Sensor:
  line_sensor_L = analogRead(line_sensor_L_pin);
  line_sensor_R = analogRead(line_sensor_R_pin);
  // print out the value you read:
  Serial.print("line_sensor_L : ");
  Serial.print(line_sensor_L);
  Serial.print(" // line_sensor_R : ");
  Serial.println(line_sensor_R);
  // 블루투스시리얼[하드웨어] 프린터 알림
  Serial1.print(line_sensor_L);
  Serial1.print(" / ");
  Serial1.println(line_sensor_R);
  //delay(1000);


  // 라인센서 아날로그 신호(백색:~100, 흑색:200~)  :: 평균값임 조도에 따라서 변화 심함
  if (line_sensor_L < line_sensor_limit_L && line_sensor_R < line_sensor_limit_R)
  {
    Serial.println("전진");
    move_Forward(PWM);
  }

  else if (line_sensor_L < line_sensor_limit_L && line_sensor_R > line_sensor_limit_R)
  {
    Serial.println("좌회전");
    move_LeftTurnRound(PWM);
  }

  else if (line_sensor_L > line_sensor_limit_L && line_sensor_R < line_sensor_limit_R)
  {
    Serial.println("우회전");
    move_RightTurnRound(PWM);
  }

  else if (line_sensor_L > line_sensor_limit_L && line_sensor_R > line_sensor_limit_R)
  {
    Serial.println("중지");
    Stop_Moter();
  }
}
// 메인 끝