/*
The Mecanum-wheel AI robot "DanVI,"
Developed by: Metaplace Co., Ltd.(T.033-252-4787)
BuiltIn(ATMEGA32U4-MU) 
PCB_ver 2.0
SMC(6850)MOTER
*/

#include <EEPROM.h>
// start reading from the first byte (address 0) of the EEPROM
int address_1 = 1;
int address_2 = 2;
byte value_1;
byte value_2;

bool prog_mode = 0;

// 모터 설정 파일을 포함해요
#include "motor_pwm_setting.h"

// 디지털핀 번호 정의
// LED가 연결된 핀 번호는 디지털 핀 13번으로 설정합니다.
#define led_pin 13
// Buzzer(부저)가 연결된 핀번호는 디지털핀 6번으로 설정합니다. 
#define buzzer_pin 6  

// 아날로그핀 번호 정의
// 푸쉬버튼이 연결된 핀 번호는 아날로그 핀 A0번으로 설정
#define button_pin A0      

// 블루투스 받는 문자열
String bluetooth_string = "";
// 버튼 상태를 저장할 변수 선언
// button_state :: 푸쉬스위치의 현재 상태를 저장하는 변수로, 0은 꺼짐(OFF), 1은 켜짐(ON)을 의미한다.
int button_state = 0;

// CW / CCW
bool Round_CW_CCW = 0;

// 라인센서 드라이브 상태
bool drive_state = 0;

// A1번 아날로그핀(line_sensor_Left_pin) 
#define line_sensor_L_pin A1          
// A2번 아날로그핀(line_sensor_Right_pin) 
#define line_sensor_R_pin A2       

// 라인센서 변수선언
int line_sensor_L, line_sensor_R;

/*
  SMC(6850) 바퀴 완조립시 rpm(목측)
  PWM(255) = 80rpm
  PWM(224) = 70rpm
  PWM(192) = 60rpm
  PWM(160) = 50rpm
*/
//위의 PWM 속도를 자유롭게 조정하여 입력해 보세요. 
// Ver 2.0(PWM 244 권장) / Ver 3.0(PWM 192 권장)
int PWM_max = 192;
int PWM_min = 160;
int PWM_step = 32;
// 라인센서 임계값
int line_sensor_limit_L = 60;
int line_sensor_limit_R = 60;

// PID 변수 선언
float Kp = 1.2;  // 비례 이득
float Ki = 0.2;  // 적분 이득
float Kd = 0.6;  // 미분 이득

float error = 0;
float previous_error = 0;
float integral = 0;
float derivative = 0;

//-----
// 부팅시 만 실행
void setup()
{
  // 시리얼 통신 초기화: 컴퓨터와의 데이터 통신 속도를 9600bps(bits per second)로 설정
  Serial.begin(9600);
  //블루투스시리얼[하드웨어] 초기화(9600)bps(bits per second)
  Serial1.begin(9600);

  // 디지털 핀 초기화: LED와 BUZZER를 제어할 핀들을 출력 모드로 설정
  pinMode(led_pin, OUTPUT);           // led_pin을 출력 모드로 설정
  pinMode(buzzer_pin, OUTPUT);        // buzzer_pin을 출력 모드로 설정    
  // motor_setting.h 파일에서 모터 핀을 설정하는 함수를 호출하여 모터 핀을 초기화
  motor_output();
  //LED 켜기 
  digitalWrite(led_pin, HIGH);  

  // EEPROM에서 값 읽기
  value_1 = EEPROM.read(address_1);
  value_2 = EEPROM.read(address_2);

  // EEPROM 기본값은 0xFF (255). 255이면 저장된 적이 없다고 가정
  if (value_1 == 255 && value_2 == 255) {
    Serial.println("No stored values in EEPROM. Using default limits.");
    EEPROM.update(address_1, line_sensor_limit_L);
    EEPROM.update(address_2, line_sensor_limit_R);
    Serial.println("Default values saved to EEPROM.");    
  } else {
    // 저장된 값을 불러오기
    line_sensor_limit_L = value_1;
    line_sensor_limit_R = value_2;
    Serial.println("Loaded values from EEPROM.");
  }
}
// 부팅시 만 실행


//-----
// 메인(반복루프)
void loop()
{
  // 블루투스 조종모드
  bluetooth_drive();

  // 라인센서 드라이브 상태
  if (drive_state == 1)
  {
    line_sensor_drive();
  }
}
// 메인 끝

void bluetooth_drive()
{
  // 블루투스 시리얼(하드웨어) 명령 수신
  if (Serial1.available()) 
  {
    if (prog_mode == 1)
    {
      bluetooth_string = Serial1.readString();  // Bluetooth로부터 문자열 읽기
      bluetooth_string.trim();  // 양쪽 공백 제거

      Serial.print("Received: ");
      Serial.println(bluetooth_string);  // 수신된 문자열을 시리얼 모니터에 출력

      // u 명령: 값을 EEPROM에 저장
      if (bluetooth_string.startsWith("u")) {
        bluetooth_string.remove(0, 1);  // 'u' 제거
        int commaIndex = bluetooth_string.indexOf(','); // 쉼표 위치 찾기

        if (commaIndex != -1) {
          // 쉼표를 기준으로 숫자 분리
          int firstValue = bluetooth_string.substring(0, commaIndex).toInt();
          int secondValue = bluetooth_string.substring(commaIndex + 1).toInt();

          // EEPROM에 값 저장
          EEPROM.update(address_1, firstValue);
          EEPROM.update(address_2, secondValue);

          // 저장 완료 메시지
          Serial.print("Saved to EEPROM: ");
          Serial.print(firstValue);
          Serial.print(", ");
          Serial.println(secondValue);
        } else {
          Serial.println("Error: Invalid format for 'u' command.");
        }
      }

      // r 명령: EEPROM에서 값을 읽어서 출력
      else if (bluetooth_string == "r") {
        // EEPROM에서 값 읽기
        byte value_1 = EEPROM.read(address_1);
        byte value_2 = EEPROM.read(address_2);

        // 읽은 값 출력
        Serial.print(address_1);
        Serial.print(" : ");
        Serial.print(value_1, DEC);
        Serial.print(" / ");
        Serial.print(address_2);
        Serial.print(" : ");
        Serial.println(value_2, DEC);

        // 블루투스시리얼[하드웨어] 프린터 알림
        Serial1.print(value_1);
        Serial1.print(" / ");
        Serial1.println(value_2);      
      }

      // l 명령: 라인센서 값을 읽어서 출력
      else if (bluetooth_string == "l") {
        // 센서 값 읽기
        line_sensor_L = analogRead(line_sensor_L_pin);
        line_sensor_R = analogRead(line_sensor_R_pin);

        Serial.print("line_sensor_L : ");
        Serial.print(line_sensor_L);
        Serial.print(" / line_sensor_R : ");
        Serial.println(line_sensor_R);

        // 블루투스시리얼[하드웨어] 프린터 알림
        Serial1.print(line_sensor_L);
        Serial1.print(" / ");
        Serial1.println(line_sensor_R);
      }    

      // v 명령: 현재 변수값 출력
      else if (bluetooth_string == "v") {

        Serial.print("line_sensor_limit_L : ");
        Serial.print(line_sensor_limit_L);
        Serial.print(" / line_sensor_limit_R : ");
        Serial.println(line_sensor_limit_R);

        // 블루투스시리얼[하드웨어] 프린터 알림
        Serial1.print(line_sensor_limit_L);
        Serial1.print(" / ");
        Serial1.println(line_sensor_limit_R);
      }          

      // d 명령: 드라이브 모드
      else if (bluetooth_string == "d") {
        prog_mode = 0;

        // EEPROM에서 값 읽기
        value_1 = EEPROM.read(address_1);
        value_2 = EEPROM.read(address_2);

        // 저장된 값을 불러오기
        line_sensor_limit_L = value_1;
        line_sensor_limit_R = value_2;
        Serial.println("Loaded values from EEPROM.");        

        Serial.println("prog_mode = 0");

        // 블루투스시리얼[하드웨어] 프린터 알림
        Serial1.println("prog_mode = 0");
      }       
      
    }
    // 해당되지 않는 경우: ASCII 값 출력 및 변환
    else if (prog_mode == 0)
    {
      { 
        // Serial1[하드웨어] 값이 있으면
        bluetooth_string = Serial1.read();    // 블루투스에서 전달된 내용 저장
        Serial.println(bluetooth_string);     //블루투스측 내용을 시리얼모니터에 출력
      }  
    }
  }

 
  // Serial.println("bluetooth_string : " + bluetooth_string);       
  //시리얼모니터에 bluetooth_string 값 출력
  if (bluetooth_string != "")         
  //bluetooth_string 값이 있다면
  { 
    // p 명령: 프로그램 모드
    if (bluetooth_string == "112") {
      prog_mode = 1;

      Serial.println("prog_mode = 1");

      // 블루투스시리얼[하드웨어] 프린터 알림
      Serial1.println("prog_mode = 1");
    }      

    // 실행(1~9)(전화버튼 기준):: ASCII
    // 1 = "49"
    if (bluetooth_string == "49")
    {
      Serial.println("라운딩 전진 좌회전");
      move_LeftTurnRoundRearAxis(PWM_max);

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      
    }

    // 2 = "50"
    else if (bluetooth_string == "50")
    {
      Serial.println("전진");
      move_Forward(PWM_max);

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      

      // 라인센서 드라이브 상태
      drive_state = 1;            
    }

    // 3 = "51"
    else if (bluetooth_string == "51")
    {
      Serial.println("라운딩 전진 우회전");
      move_RightTurnRoundRearAxis(PWM_max);

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      
    }

    // 4 = "52"
    else if (bluetooth_string == "52")
    {
      Serial.println("좌측이동");
      move_Leftward(PWM_max);

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      
    }

    // 5 = "53"
    else if (bluetooth_string == "53")
    {
      Serial.println("중지");
      Stop_Moter();

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      

      // 라인센서 드라이브 상태
      drive_state = 0;      
    }
    
    // 6 = "54"
    else if (bluetooth_string == "54")
    {
      Serial.println("우측이동");
      move_Rightward(PWM_max);    

      // BUZZER OFF: buzzer_pin(6번 핀)에 LOW 신호를 보내 부저(Buzzer)를 끕니다.
      digitalWrite(buzzer_pin, LOW);
      // 시리얼 모니터에 "BUZZER OFF" 메시지를 출력합니다.
      Serial.println("BUZZER OFF");      
    }

    // 7 = "55"
    else if (bluetooth_string == "55")
    {
      Serial.println("라운딩 후진 좌회전");
      move_LeftTurnRoundFrontAxis(PWM_max);

      // BUZZER ON: buzzer_pin(6번 핀)에 HIGH 신호를 보내 부저(Buzzer)를 켭니다.
      digitalWrite(buzzer_pin, HIGH);
      // 시리얼 모니터에 "BUZZER ON" 메시지를 출력합니다.
      Serial.println("BUZZER ON");    
    }

    // 8 = "56"
    else if (bluetooth_string == "56")
    {
      Serial.println("후진");
      move_Backward(PWM_max);

      // BUZZER ON: buzzer_pin(6번 핀)에 HIGH 신호를 보내 부저(Buzzer)를 켭니다.
      digitalWrite(buzzer_pin, HIGH);
      // 시리얼 모니터에 "BUZZER ON" 메시지를 출력합니다.
      Serial.println("BUZZER ON");      
    }

    // 9 = "57"
    else if (bluetooth_string == "57")
    {
      Serial.println("라운딩 후진 우회전");
      move_RightTurnRoundFrondAxis(PWM_max);

      // BUZZER ON: buzzer_pin(6번 핀)에 HIGH 신호를 보내 부저(Buzzer)를 켭니다.
      digitalWrite(buzzer_pin, HIGH);
      // 시리얼 모니터에 "BUZZER ON" 메시지를 출력합니다.
      Serial.println("BUZZER ON");          
    }

    // 0 = "48"
    else if (bluetooth_string == "48")
    {
      Serial.println("라운딩 회전");
      // CW / CCW
      if (Round_CW_CCW == 0)
      {
        RoundCenterAxisCW(PWM_max);
        Round_CW_CCW = 1;
      }
      else if (Round_CW_CCW == 1)
      {
        RoundCenterAxisCCW(PWM_max);
        Round_CW_CCW = 0;
      }

      // BUZZER ON: buzzer_pin(6번 핀)에 HIGH 신호를 보내 부저(Buzzer)를 켭니다.
      digitalWrite(buzzer_pin, HIGH);
      // 시리얼 모니터에 "BUZZER ON" 메시지를 출력합니다.
      Serial.println("BUZZER ON");          
    }

    // * = "42"
    // * 버튼을 누를 때마다 PWM_step만큼 모터 속도가 줄어듭니다.
    else if (bluetooth_string == "42")
    {
      // "moter_down"이라는 메시지를 시리얼 모니터에 출력합니다.
      Serial.println("moter_down");

      // 모터 속도를 줄입니다. 현재 속도에서 PWM_step만큼 감소시킵니다.
      PWM_max -= PWM_step;
      // 모터의 최소 속도를 160으로 제한합니다. PWM이 160 미만으로 내려가지 않도록 설정합니다.
      if (PWM_max < 160)
      {
        PWM_max = 160;
      }

     // 현재 PWM 값을 시리얼 모니터에 출력합니다.
      Serial.println(PWM_max);
      // 현재 PWM 값을 블루투스 시리얼에 출력합니다.
      Serial1.println(PWM_max);
    }
    
    // # = "35"
    else if (bluetooth_string == "35")
    {
      // 블루투스에서 "35" 문자열을 수신하면, "motor_up"이라는 메시지를 시리얼 모니터에 출력합니다.
      Serial.println("motor_up");

      // 모터 속도를 증가시킵니다. 현재 속도에 PWM_step만큼 더합니다.
      PWM_max += PWM_step;
      // 모터의 최대 속도를 255로 제한합니다. PWM이 255를 초과하지 않도록 설정합니다.
      if (PWM_max > 255)
      {
       PWM_max = 255;
      }

      // 현재 PWM 값을 시리얼 모니터에 출력합니다.
      Serial.println(PWM_max);
      // 현재 PWM 값을 블루투스 시리얼에 출력합니다.
      Serial1.println(PWM_max);
    }  
   
  // bluetooth_string초기화
  bluetooth_string = "";
  }

  // 시리얼통신 명령 수신
  if (Serial.available()) {
    Serial1.write(Serial.read());  // 시리얼모니터 내용을 블루투스측에 출력
  }  
}


void line_sensor_drive()
{
  // 센서 값 읽기
  line_sensor_L = analogRead(line_sensor_L_pin);
  line_sensor_R = analogRead(line_sensor_R_pin);

  // 에러 계산 (라인의 중심에서 얼마나 벗어났는지 계산)
  error = line_sensor_R - line_sensor_L;  

  // PID 계산
  integral += error;  // 적분항 누적
  integral = constrain(integral, -30, 30); // 적분 제한 (더 낮게 조정)
  derivative = error - previous_error;  // 미분항 계산

  float correction = Kp * error + Ki * integral + Kd * derivative;
  previous_error = error;  // 이전 에러 업데이트

  // 모터 PWM 값 계산
  int Motor_PWM_L = PWM_max - correction;
  int Motor_PWM_R = PWM_max + correction;

  // 모터 PWM 제한
  Motor_PWM_L = constrain(Motor_PWM_L, PWM_min, PWM_max); // 최소 속도 100으로 설정
  Motor_PWM_R = constrain(Motor_PWM_R, PWM_min, PWM_max);

  // 시리얼 출력
  Serial.print("line_sensor_L : ");
  Serial.print(line_sensor_L);
  Serial.print(" // line_sensor_R : ");
  Serial.println(line_sensor_R);

  Serial.print("Error: ");
  Serial.print(error);
  Serial.print(" | Correction: ");
  Serial.print(correction);
  Serial.print(" | Motor_PWM_L: ");
  Serial.print(Motor_PWM_L);
  Serial.print(" | Motor_PWM_R: ");
  Serial.println(Motor_PWM_R);  

  // 블루투스시리얼[하드웨어] 프린터 알림
  Serial1.print(line_sensor_L);
  Serial1.print(" / ");
  Serial1.println(line_sensor_R);
  //delay(1000);


  // 모터 제어 로직
  if (line_sensor_L < line_sensor_limit_L && line_sensor_R < line_sensor_limit_R) {
      move_Forward_for_line(Motor_PWM_L, Motor_PWM_R);
  } else if (line_sensor_L < line_sensor_limit_L && line_sensor_R > line_sensor_limit_R) {
      // PID 기반 좌회전
      float turn_correction = Kp * (line_sensor_R - line_sensor_L);
      int Turn_PWM_L = turn_correction;

      Turn_PWM_L = constrain(Turn_PWM_L, PWM_min, PWM_max);

      Serial.print("Turn_PWM_L: ");
      Serial.println(Turn_PWM_L);

      Serial.println("PID 기반 좌회전");
      move_LeftTurnRound(Turn_PWM_L);
  } else if (line_sensor_L > line_sensor_limit_L && line_sensor_R < line_sensor_limit_R) {
      // PID 기반 우회전
      float turn_correction = Kp * (line_sensor_L - line_sensor_R);
      int Turn_PWM_R = turn_correction;

      Turn_PWM_R = constrain(Turn_PWM_R, PWM_min, PWM_max);

      Serial.print("Turn_PWM_R: ");
      Serial.println(Turn_PWM_R);

      Serial.println("PID 기반 우회전");
      move_RightTurnRound(Turn_PWM_R);
  } else {
      Serial.println("중지");
      Stop_Moter();
  }  
}